##############################################
### CSSS/POLS 512 Sp 2022                    
### University of Washington
### Prof. Chris Adolph
### TA. Tao Lin

### Lab 1 Code
### Logistics & R Refresher
### April 1, 2022
##############################################

setwd(Your.Current.Working.Directory)

##### A simple example to generate minimal reproducible example #####

# install.packages("reprex")
# install.packages("datapasta")
library(reprex)
library(datapasta)

tribble_paste(mtcars) # if you have a small data.frame, you can use it

reprex(
  {
    f <- function(x) x + 1 # create f
    g <- function(x) f(x)  # create g
    g("a")
    ggplot()
  },
  session_info = TRUE, # include information about your computer and current R session (important!)
  style = TRUE, # make your code prettier
  venue = "slack" # choose slack style
)

##### Coding Exercise 1: Time Series Data - Global Temperature #####

# install.packages("tidyverse")
library(tidyverse)

# Acquire the data
# Average global temperature from Univ. East Anglia and UK Met Office
# Monthly from January 1856 to December 2005
global <- read.table("Lab1_data/global.dat")
global <- t(global) |> as.vector() |> as.data.frame()

# 1. Create a time series object using the data that starts in Jan 1856 and ends in Dec 2005 with monthly observations.

global.month.ts <- ts(global, start = c(1856, 1), end = c(2005, 12), frequency = 12)

# 2. Find the mean temperature for each year and save in a new time series object.

global.annual.ts <- aggregate(global.month.ts) / 12

# 3. Plot the two objects.
plot(global.month.ts)
plot(global.annual.ts)

# 4. Observe global temperature from 1970 to 2005 using the window function and plot.

global.window <- window(global.annual.ts, start = 1970, end = 2005)
plot(global.window)

##### Coding Exercise 2: Panel Data - Democracy and Income #####

data <- read.csv("Lab1_data/Lab1data.csv", header = TRUE)
colnames(data) <- c("country", "Year", "income", "polity2")
head(unique(data$country)) # observations on 174 countries
head(tapply(data$country, data$Year, length)) # inspect how many countries in each year
head(tapply(data$Year, data$country, length)) # inspect how many years each country has
nrow(data)

p <- ggplot(data = na.omit(data), aes(x = Year, y = income, 
                                      group = country, color = country))
p + geom_line(alpha = 0.5) + # make lines semi-transparent
  guides(color = FALSE) # turn off the legend because there are so many countries

# 1. Subset the data frame to show only country name and GDP per capita
data %>%
  select(country, income)

# 2. Rearrange the columns of the data frame ascending by polity score
data %>%
  arrange(polity2) # how to make it in a descending order?

# 3. Show only values of GDP per capita for South Africa from 2002 to 2008
data %>%
  subset(country == "South Africa" & Year >= 2002 & Year <= 2008)

# 4. Create a new variable that takes the first letter of the country and attaches it to the year of observation
data %>%
  mutate(
    new.var = substr(country, 1, 1) %>% # you can even use pipe within `dplyr` function
      paste0(Year, .)
  )

# 5. Find the mean of GDP per capita for each year of observation

tapply(data$income, data$Year, mean, na.rm = TRUE) # why 2010 is NA?

##############################################
# A simple example for debugging Functions
##############################################
dat <- data.frame(
  row.names = c("calories", "weight", "yumminess"),
  blackberry = c(4L, 9L, 6L),
  blueberry = c(1L, 2L, 8L),
  peach = c(59L, 150L, 10L),
  plum = c(30L, 78L, 5L)
)

fruit_avg <- function(dat, pattern) {
  cols <- grep(pattern, names(dat))
  mini_dat <- dat[ , cols, drop = FALSE] # coerce a single column to be a matrix, otherwise it will be a vector and cause an error; but if `dat` is a tibble, we don't need to do that. To see why, check https://adv-r.hadley.nz/vectors-chap.html#safe-subsetting.
  message("Found ", ncol(mini_dat), " fruits!")
  rowMeans(mini_dat)
}

dat
fruit_avg(dat, pattern = "berry")
fruit_avg(dat, pattern = "black")
debugonce(fruit_avg)
fruit_avg(dat, pattern = "black")